#ifndef __INC_BPLA_KERNEL_H__
#define __INC_BPLA_KERNEL_H__

#include <string>
#include <boost/multi_array.hpp>
#include <boost/program_options.hpp>

#include "Utility.h"
#include "kernel/common/rna.h"

class BPLAKernel
{
public:
  enum Mode {DATABASE, QUERY};

public:
  struct Options
  {
    bool noBP;
    double gap;
    double ext;
    double alpha;
    double beta;
    std::string score_file;

    Options() : noBP(false), gap(-27), ext(-0.1), alpha(1.0) {}
    Options(Mode mode);

    void add_options(boost::program_options::options_description& desc);
  };

public:
  BPLAKernel(const Options& opts);
  BPLAKernel() : score_table_(boost::extents[N_RNA][N_RNA]) {}
  ~BPLAKernel() {}

  double operator()(const FeatureVector& x, const FeatureVector& y,
		    bool rescale, double& scale) const;
  double operator()(const FeatureVector& x, const FeatureVector& y) const;

private:
  void read_score_table(const std::string& score_file);

private:
  boost::multi_array<double,2> score_table_;
  bool noBP_;
  double gap_;
  double ext_;
  double alpha_;
  double beta_;

private:
  friend class boost::serialization::access;
  template <class Archive> 
  void serialize(Archive& ar, const unsigned int ver)
  {
    for (uint i=0; i!=N_RNA; ++i) 
      for (uint j=0; j!=N_RNA; ++j) 
        ar & score_table_[i][j];

    ar & noBP_ & gap_ & ext_ & alpha_ & beta_;
  }
};

#endif
