#ifndef __INC_DATA_H__
#define __INC_DATA_H__

#include <vector>
#include <list>
#include <string>
#include <fstream>
#include <boost/spirit/iterator/file_iterator.hpp>

namespace Vienna {
  extern "C" {
#include <ViennaRNA/utils.h>
  };
};

#include "Utility.h"
#include "kernel/common/bpmatrix.h" 

struct FeatureSequence
{
  typedef std::list<std::string> IS;

  std::vector<FeatureVector> feat;
  std::string name;

  FeatureSequence() : feat(), name() {}
  FeatureSequence(const FeatureSequence& x) : feat(x.feat), name(x.name) {}

  FeatureSequence(const IS& seq);
  FeatureSequence(const IS& seq, float pf_scale, const BPMatrix::Options& opts);

  int size() const { return feat.size(); }
};

class DataLoader
{
public:
  typedef FeatureSequence Data;

public:
  DataLoader(const char* filename,
	     const BPMatrix::Options& bp_opts, bool use_bp);
  DataLoader(const char* filename, const char* pf_scales,
	     const BPMatrix::Options& bp_opts, bool use_bp);
  ~DataLoader();

  Data* get();

private:
  const BPMatrix::Options& bp_opts_;
  bool use_bp_;
  std::string filename_;
  int type_;
  boost::spirit::file_iterator<> fi_;
  std::ifstream* pf_is_;
};

class DataLoaderFactory
{
public:
  typedef DataLoader Loader;
  typedef Loader::Data Data;

public:
  DataLoaderFactory(const BPMatrix::Options& bp_opts)
    : bp_opts_(bp_opts), use_bp_(true)
  {
    Vienna::init_rand();
  }

  DataLoaderFactory() : use_bp_(false) {}

  Loader* get_loader(const char* filename) const
  {
    return new Loader(filename, bp_opts_, use_bp_);
  }

  Loader* get_loader(const char* filename, const char* pf_scales) const
  {
    return new Loader(filename, pf_scales, bp_opts_, use_bp_);
  }

private:
  BPMatrix::Options bp_opts_;
  bool use_bp_;
};

#endif
