#include <string>
#include <iostream>
#include <cassert>
#include <boost/program_options.hpp>
#include <boost/bind.hpp>

#include "FrameworkKind.h"
#include "KernelizedLSH.h"
#include "ForestLSH.h"
#include "BPLAKernel.h"
#include "Data.h"
#include "kernel/common/bpmatrix.h"

using namespace std;
using namespace boost;
namespace po = program_options;

int 
main(int argc, char** argv)
{
  Options opts;
  LSH::Options lsh_opts;
  BPLAKernel::Options knl_opts(BPLAKernel::DATABASE);
  BPMatrix::Options fld_opts;

  // parse command line options
  po::options_description desc("Options");
  desc.add_options()("help,h", "show this message");
  opts.add_options(desc);

  po::options_description l_desc("LSH Options");
  lsh_opts.add_options(l_desc);
  po::options_description k_desc("Kernel Options");
  knl_opts.add_options(k_desc);
  //po::options_description f_desc("Folding Options");
  //fld_opts.add_options(f_desc);
  
  desc.add(l_desc).add(k_desc);
  //desc.add(l_desc).add(k_desc).add(f_desc);
  po::variables_map vm;
  po::parsed_options parsed =
    po::command_line_parser(argc, argv).
    options(desc).allow_unregistered().run();
  vector<string> extra_args =
    collect_unrecognized(parsed.options, po::include_positional);
  vector<po::option>::iterator new_end =
    remove_if(parsed.options.begin(), parsed.options.end(),
	      bind(&po::option::unregistered, _1) );
  parsed.options.erase(new_end, parsed.options.end());
  po::store(parsed, vm);
  po::notify(vm);
  
  if (vm.count("help") || extra_args.size()<2) {
    cout << "Kind - create a kernel index database" << endl
	 << endl
	 << "Usage:" << endl
	 << "  " << argv[0] << " [options] output seq.mfa" << endl
	 << endl
	 << desc << endl;
    return 1;
  }

  opts.parse_extra_args(extra_args);

  typedef BPLAKernel KNL; 
  typedef DataLoaderFactory LDF;
  KNL knl(knl_opts);
  LDF ldf(fld_opts);

  bool res = false;
  try {
    if (opts.forest) {
      typedef ForestLSH<KNL> LSH;
      LSH lsh(lsh_opts, opts.ntrn, opts.nsmp, opts.rescale, knl);
      App<LSH,LDF> app(lsh,ldf,opts);
      res = app.execute();      
    }
    else {
      typedef KernelizedLSH<KNL> LSH;
      LSH lsh(lsh_opts, opts.ntrn, opts.nsmp, opts.rescale, knl);
      App<LSH,LDF> app(lsh,ldf,opts);
      res = app.execute();      
    }
  } 
  catch (const char* str) {
    std::cout << str << std::endl;
  }

  return res ? 0 : 1;
}
