#ifndef __INC_UTILITY_H__
#define __INC_UTILITY_H__

#include <utility>
#include <vector>
#include <functional>
#include <boost/random.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/io.hpp>

#define PI 3.141592653589793238462643383279502884197169399375105820974944

typedef float FeatureValue; 
typedef std::vector<FeatureValue> FeatureVector;
typedef boost::mt19937 RandomGenerator;

enum { 
  FEAT_SEQ_A = 0, 
  FEAT_SEQ_C = 1, 
  FEAT_SEQ_G = 2, 
  FEAT_SEQ_U = 3,
  FEAT_STR_L = 4,
  FEAT_STR_R = 5,
  FEAT_STR_U = 6,
  N_SEQ_FEAT = 4,
  N_STR_FEAT = 3,
  N_FEAT = 7,
};

struct RandomShuffler {
  RandomGenerator& rng_;

  RandomShuffler(RandomGenerator& rng) : rng_(rng) {}

  unsigned operator()(unsigned i);

};

unsigned long int 
intpow(const int a, const int b);

double 
dblpow(const int a, const int b);

FeatureValue 
sqdist(const FeatureVector& x, const FeatureVector& y);

FeatureValue 
dist(const FeatureVector& x, const FeatureVector& y);

FeatureVector
normalized_vector(const FeatureVector& x);

std::vector<double> 
random_normal_vector(const uint ndim, RandomGenerator& rng);

boost::numeric::ublas::matrix<double> 
random_rotation_matrix(const uint ndim, RandomGenerator& rng);

void 
pca(const std::vector<FeatureVector>& dat, 
    boost::numeric::ublas::matrix<double>& pc, std::vector<double>& cr);

std::vector<std::vector<double> >
sqrt_symmetric_matrix(const std::vector<std::vector<double> >& mat);

#endif
